@@ -161,6 +161,7 @@ normalise_swc<-function(x, requiredColumns=c('PointNo','Label','X','Y','Z','W','
 #'   [SubTrees]) NB SubTrees will only be present when nTrees>1.
 #' @export
 #' @method as.neuron ngraph
+#' @importFrom igraph V V<- vcount decompose.graph
 #' @rdname neuron
 #' @seealso \code{\link{graph.dfs}, \link{as.seglist}}
 as.neuron.ngraph<-function(x, vertexData=NULL, origin=NULL, Verbose=FALSE, ...){
@@ -175,7 +176,7 @@ as.neuron.ngraph<-function(x, vertexData=NULL, origin=NULL, Verbose=FALSE, ...){
   # save original vertex ids
   igraph::V(x)$vid=seq.int(igraph::vcount(x))
   # check if we have multiple subgraphs
-  if(no.clusters(x)>1){
+  if(igraph::no.clusters(x)>1){
     if(!length(origin)){
       # no origin specified, will pick the biggest subtree
       # decompose into list of subgraphs
@@ -331,21 +332,23 @@ as.neuron.default<-function(x, ...){
 #' @export
 `/.neuron` <- function(n,x) n*(1/x)
 
-#' Divide neuron coords by a factor (and optionally center)
-#'
-#' @details Note that if scale=TRUE, the neuron will be rescaled to unit sd in each axis
-#' likewise if center=TRUE, the neuron will be centred around the axis means
+#' Scale and centre neuron 3D coordinates
+#' 
+#' @details If \code{scale=TRUE}, the neuron will be rescaled to unit sd in each
+#'   axis. If \code{center=TRUE}, the neuron will be centred around the axis
+#'   means. See \code{base::\link{scale.default}} for additional details.
 #' @param x A neuron
 #' @param center 3-vector to subtract from x,y,z coords
 #' @param scale 3-vector used to divide x,y,z coords
 #' @return neuron with scaled coordinates
 #' @method scale neuron
 #' @export
-#' @seealso \code{\link{scale.default}}
+#' @seealso \code{\link{scale.default}}, \code{\link{*.neuron}}
+#' @aliases scale
 #' @examples
 #' n1.scaledown=scale(Cell07PNs[[1]],scale=c(2,2,3))
 #' n1.scaleup=scale(Cell07PNs[[1]],scale=1/c(2,2,3))
-scale.neuron<-function(x,center=FALSE,scale=FALSE){
+scale.neuron<-function(x, center=TRUE, scale=TRUE){
   xyzmatrix(x)<-scale(xyzmatrix(x),scale=scale,center=center)
   x
 }
@@ -509,6 +512,7 @@ resample.neuron<-function(x, stepsize, ...) {
 # Interpolate ordered 3D points (optionally w diameter)
 # NB returns NULL if unchanged (when too short or <=2 points) 
 # and only returns _internal_ points, omitting the head and tail of a segment
+#' @importFrom stats approx
 resample_segment<-function(d, stepsize, ...) {
   # we must have at least 2 points to resample
   if(nrow(d) < 2) return(NULL)
